// Copyright (c) Mito
import React, { useRef, useState } from 'react';
import "../../../../css/taskpanes/ControlPanel/ControlPanelTaskpane.css";
import { useDebouncedEffect } from '../../../hooks/useDebouncedEffect';
import { getCellDataFromCellIndexes } from '../../endo/utils';
import { TaskpaneType } from '../taskpanes';
import ColumnNameCard from './ColumnNameCard';
import ControlPanelTaskpaneTabs from './ControlPanelTaskpaneTabs';
import DtypeCard from './FilterAndSortTab/DtypeCard';
import FilterCard from './FilterAndSortTab/filter/FilterCard';
import { isFilterGroup } from './FilterAndSortTab/filter/filterTypes';
import { isValidFilter, parseFilter } from './FilterAndSortTab/filter/utils';
import SortCard from './FilterAndSortTab/SortCard';
import ColumnSummaryGraph from './SummaryStatsTab/ColumnSummaryGraph';
import ColumnSummaryStatistics from './SummaryStatsTab/ColumnSummaryStatistics';
import { ValuesTab } from './ValuesTab/ValuesTab';
import FormatCard from './FilterAndSortTab/FormatCard';
/*
    We wait 500ms before sending a filter message to make sure
    that as the user is typing key changes, we don't queue up a
    ton of filtering messages.
*/
const FILTER_MESSAGE_DELAY = 500;
export var ControlPanelTab;
(function (ControlPanelTab) {
    ControlPanelTab["FilterSort"] = "filter_sort";
    ControlPanelTab["UniqueValues"] = "unique_values";
    ControlPanelTab["SummaryStats"] = "summary_stats";
})(ControlPanelTab || (ControlPanelTab = {}));
export const ControlPanelTaskpane = (props) => {
    // Get the values for the last cell that was selected
    const { columnHeader, columnID, columnFormula, columnMitoType, columnFilters } = getCellDataFromCellIndexes(props.sheetData, props.selection.endingRowIndex, props.selection.endingColumnIndex);
    const [filters, setFilters] = useState(columnFilters !== undefined ? columnFilters.filters : []);
    const [operator, setOperator] = useState(columnFilters !== undefined ? columnFilters.operator : 'And');
    const [stepID, setStepID] = useState('');
    // If this is not a valid column, don't render anything, and close the takspane! 
    // We have to do this after the useState calls, to make sure this is valid react
    if (columnHeader === undefined || columnID === undefined || columnFormula === undefined || columnMitoType === undefined) {
        props.setUIState((prevUIState) => {
            return Object.assign(Object.assign({}, prevUIState), { currOpenTaskpane: { type: TaskpaneType.NONE } });
        });
        return React.createElement(React.Fragment, null);
    }
    // When the filters or operator changes, send a new message, as long as this is not
    // the first time that this rendered. We use a ref to avoid sending a message the first 
    // time it renders
    const firstRender = useRef(true);
    useDebouncedEffect(() => {
        if (!firstRender.current) {
            void _sendFilterUpdateMessage();
        }
        firstRender.current = false;
    }, [filters, operator], FILTER_MESSAGE_DELAY);
    /*
        NOTE: only call this through the sendFilterUpdateMessage function, to make sure
        buffering messages works properly.

        Before sending the displayed filters, we parse all the number filters from strings
        to numbers, and then filter out all of the invalid filters (as to not cause errors)
    */
    const _sendFilterUpdateMessage = async () => {
        // To handle decimals, we allow decimals to be submitted, and then just
        // parse them before they are sent to the back-end
        const parsedFilters = filters.map((filterOrGroup) => {
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.map((filter) => {
                        return parseFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return parseFilter(filterOrGroup);
            }
        });
        const filtersToApply = parsedFilters.map((filterOrGroup) => {
            // Filter out these incomplete filters from the group
            if (isFilterGroup(filterOrGroup)) {
                return {
                    filters: filterOrGroup.filters.filter((filter) => {
                        return isValidFilter(filter);
                    }),
                    operator: filterOrGroup.operator
                };
            }
            else {
                return filterOrGroup;
            }
        }).filter((filterOrGroup) => {
            // Filter out the groups if they have no valid filters in them
            if (isFilterGroup(filterOrGroup)) {
                return filterOrGroup.filters.length > 0;
            }
            // And then we filter the non group filters to be non-empty
            return isValidFilter(filterOrGroup);
        });
        const _stepID = await props.mitoAPI.sendFilterMessage(props.selectedSheetIndex, columnID, filtersToApply, operator, stepID, props.tab);
        setStepID(_stepID);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { className: 'control-panel-taskpane-container default-taskpane-div' },
            React.createElement(ColumnNameCard, { columnHeader: columnHeader, columnIndex: props.selection.endingColumnIndex, mitoContainerRef: props.mitoContainerRef, gridState: props.gridState, setEditorState: props.setEditorState, mitoAPI: props.mitoAPI, setUIState: props.setUIState }),
            React.createElement("div", { className: 'default-taskpane-body-div mb-30px' },
                props.tab === ControlPanelTab.FilterSort &&
                    React.createElement(React.Fragment, null,
                        React.createElement(DtypeCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, columnFormula: columnFormula, mitoAPI: props.mitoAPI, lastStepIndex: props.lastStepIndex, lastStepType: props.lastStepType }),
                        React.createElement(FormatCard, { columnID: columnID, mitoAPI: props.mitoAPI, gridState: props.gridState, columnMitoType: columnMitoType, sheetData: props.sheetData }),
                        React.createElement(SortCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI }),
                        React.createElement(FilterCard, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, filters: filters, setFilters: setFilters, setOperator: setOperator, columnMitoType: columnMitoType, operator: operator, mitoAPI: props.mitoAPI })),
                props.tab === ControlPanelTab.UniqueValues &&
                    React.createElement(React.Fragment, null,
                        React.createElement(ValuesTab, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, filters: filters, setFilters: setFilters, mitoAPI: props.mitoAPI, columnMitoType: columnMitoType })),
                props.tab === ControlPanelTab.SummaryStats &&
                    React.createElement(React.Fragment, null,
                        React.createElement(ColumnSummaryGraph, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI }),
                        React.createElement(ColumnSummaryStatistics, { selectedSheetIndex: props.selectedSheetIndex, columnID: columnID, mitoAPI: props.mitoAPI })))),
        React.createElement(ControlPanelTaskpaneTabs, { selectedTab: props.tab, setUIState: props.setUIState, mitoAPI: props.mitoAPI })));
};
export default ControlPanelTaskpane;
//# sourceMappingURL=ControlPanelTaskpane.js.map